% HU Berlin x-ray ImageJ plugins
% Linus Pithan, AG Kowarik
% Version 0.1 beta, April 15, 2015

Introduction
============

This software package is designed to import and analyze pixel detector image files from x-ray diffraction measurements. 

Installation
============

 - Make sure that you have a copy of [ImageJ], [ImageJ2] or [Fiji] installed on your system.
 - Find the installation directory of *ImageJ* on your system and locate the *plugin* folder.
 - Copy the folder *hu-berlin-xray* into the *ImageJ* plugin directory.
 - Check if there is a new entry *hu-berlin-xray* in the *Plugins* menu when starting *ImageJ* (in case it didn't work please have a look at [imagejdocu])

Usage
=====

Within this package there are different independent plugins for distinct tasks. 
 - The `EDF Stack import` plugin is designed to load stacks of EDF files  (“ESRF Data Format”)  into *ImageJ* while making use of the EDF file metadata header that contains information e.g. on motor positions or spec counters.
 - The `Stack Stitcher` plugin helps to stitch a stack of images (e.g. resulting from an GIDX scan, where only the detector is moved). 
 - The `Moving ROI through Stack` plugin helps to extract the integrated density in a region of interest (ROI) that moves on a linear trajectory through a stack.

EDF Stack import
----------------

![import dialog](screenshots/EDF_stack_import/importDialog.png )

![perform measurements within the stack](screenshots/EDF_stack_import/setMeasurements_plotProfile.png )

A sample dataset is provided in the folder `example_datasets/sample_EDF_stack`.
 
 - Select `Plugins/hu-berlin-xray/EDF Stack import` from the ImageJ menu bar.
 - Choose the directory that contains the EDF files that you want to import (Warning: make sure that this directory ONLY contains EDF files, otherwhise it won't work)
 - A dialog window (screenshot) opens and you are able to select how the metadata of each image file should be used during the file import. E.g. You may want to normalize all files to the storage ring current or make use of calibrated absorbers. Additionally it is possible to directly use motor positions as lables within the stack, this is especially useful if your stack contains images resulting from a motor scan.

To analyze the data in the stack you may want to sum up the counted intensity in a certain region for all sides. To do so, just select the region and and go to `Plot z-axis Profile` in the menu under the `Stk` button (screenshot). This will open both a plot an a result table that you can export and use for futher processing. The column *Label* contains the information that has been secelected during the import of the stack. The column *RawIntDen* contains the sum over all pixels in the selected region. (You may have to go to `Analyze/Set Measurements...` to set these two coulmns (screenshot))

Stack Stitcher
--------------

![Stack Stitcher plugin](screenshots/stack_stitcher.png )

This plugin requires stacks of images that can be put together in a way that the first image in the stack has to be the leftmost image. A sample image stack is provided in the file `example_datasets/gixd_scan_64px_overlay.tif`.

 - Open a stack of image files that you want to stitch together.
 - Select `Plugins/hu-berlin-xray/Stack Stitcher` from the ImageJ menu bar.
 - A dialog (screenshot) pops up where you are most importantly asked to enter a *Overlay* in pixel. This Overlay is defined by the amount of pixel that a subsequent picture overlaps the preceding one (For the provided example dataset this is 64px). In most cases all the other parameters in the dialog will adjust automatically.

Moving ROI xy-Position through stackdepth
=========================================

![Moving ROI Position through Stack](screenshots/MovingROI.png )

This plugin requires stacks of images as e.g. `example_datasets/rocking_scan.tif`. Based on the position of a cental pixel in the first and last image, this plugin moves a ROI position linearly as a function of the stack depth and acquires the integrated density in the ROI for each slice

 - Open a stack and pick the central pixel position in the first and last slide of the stack (e.g. in the `rocking_scan.tif` *first slice: x=33 y=93; last slice x=33 y=131).
 - Select `Plugins/hu-berlin-xray/Moving ROI through Stack` from the ImageJ menubar.
 - Enter the 4 coordinates as well as the size of the ROI (box with x and y) that you want to use
 - When pressing OK you can visually follow the path of the ROI with your eye and a resut table will pop up (screenshot). 
 - In case you want to speed up the process set the sleeptime to 0 (this results in a lack of visual control).

Sources
=======

The `EDF Stack import` plugin is partially based on the [ESRF image format reader] code by Olof Svensson (svensson@esrf.fr) and Petr Mikulik (mikulik@physics.muni.cz).


Known Bugs
==========

EDF Stack import
----------------
 - Folders may ONLY contain .EDF files, otherwhise the import wont work

Licence and Citation
====================
If you want to do the author a favour please cite *L. Pithan, C. Cocchi, H. Zschiesche, C. Weber, A. Zykov, S. Bommel, S. J. Leake, P. Schäfer, C. Draxl, S. Kowarik, “Light control of polymorphism in thin films of Sexithiophene”
Crystal Growth and Design, 15 (3), 1319-1324, (2015),* [doi:10.1021/cg501734w]

The software is provided under the *MIT License*, see `LICENSE` file for details.

Contact
=======
Please do not hesitate to contact linus.pithan@physik.hu-berlin.de for assistance.

Version History
===============
Still first beta version...

[ImageJ]: http://imagej.nih.gov/ij/
[ImageJ2]: http://fiji.sc/ImageJ2
[Fiji]: http://fiji.sc/Fiji

[imagejdocu]: http://imagejdocu.tudor.lu/doku.php?id=howto:plugins:how_to_install_a_plugin

[ESRF image format reader]: https://github.com/olofsvensson/esrf-imagej-plugins

[doi:10.1021/cg501734w]: http://pubs.acs.org/doi/abs/10.1021/cg501734w
