/*
***********************************************************************
* Written and maintained by Linus Pithan 			      *
* (linus.pithan@physik.hu-berlin.de)                                  *   
*                                                                     *
* Based on the ESRF "image format reader" code pubished at	      *
* https://github.com/olofsvensson/esrf-imagej-plugins                 *
* by Olof Svensson (svensson@esrf.fr) and 			      *
* Petr Mikulik (mikulik@physics.muni.cz)	    		      *
*                                            			      *
* Changes:                                                            *
*                                                                     *
* 15.06.2015  Support for stacks without metadata                     *
* 15.04.2015  Finalisation of version 0.1                             *
* 01.12.2014  Support for SignedLong Maxipix files from ID3           *
*                                                                     *
***********************************************************************
*/

import java.io.*;
import java.util.*;

import ij.*;
import ij.io.*;
import ij.gui.*;
import ij.plugin.PlugIn;
import ij.process.*;


/** This plugin reads image formats used at the ESRF **/
public class EDF_Stack_import_ implements PlugIn {

  private static final String EDF_Stack_import_VERSION = "v0.1 April 2015";

  private boolean metadata= false;

  // Default values for an image
  private int width = 512;
  private int height = 512;
  private int offset = 0;
  private int nImages = 1;
  private int gapBetweenImages = 0;
  private boolean whiteIsZero = false;
  private boolean intelByteOrder = false;
  private boolean openAll = false;
  private static String defaultDirectory = null;
  private int fileType = FileInfo.GRAY16_UNSIGNED;
  private int transmfactor=1; // Added 2014
	
  private ArrayList counterList,counterValueList,motorList,motorValueList;
	

  public void run(String arg)
  { 
	
    String directory, fileName, type = "EDF";
    OpenDialog od;
    FileOpener fileOpener;
    File f;
    directory =(new DirectoryChooser("Select folder containing EDF files")).getDirectory();
    File dir=new File(directory);
    String[] children = dir.list();
    if (children[0]==null) return;

    {
    f = new File(directory,children[0]);
    FileInfo fileInfo = new FileInfo();
    fileInfo.fileFormat = fileInfo.RAW;
    fileInfo.fileName = children[0];
    fileInfo.directory = directory;
    fileInfo.width = width;
    fileInfo.height = height;
    fileInfo.offset = offset;
    fileInfo.nImages = nImages;
    fileInfo.gapBetweenImages = gapBetweenImages;
    fileInfo.intelByteOrder = intelByteOrder;
    fileInfo.whiteIsZero = whiteIsZero;
    fileInfo.fileType = fileType ;
    parseESRFDataFormatHeader("EDF", f, fileInfo);
    width=fileInfo.width;
    height=fileInfo.height;
    }

	String[] counters={""};
	counters=(String[])counterList.toArray(counters);
	String[] motors = {""};
	motors=(String[])motorList.toArray(motors);
	// String counter1="",counter2="",counter3="",zmotor="",zcounter="";
	int counter1, counter2, counter3, zmotor, zcounter;
	String counter1RB,counter2RB,counter3RB,zRB;
	String[] RBlabels={"Multiply","Divide","Don't use"};
	String[] RBZlabels={"Filename","Motor","Counter"};
	
	metadata=counters[0] != null & motors[0] != null;

	//IJ.write("metadata : "+motors[0]);

	  GenericDialog gd = new GenericDialog("Stack Normalization", IJ.getInstance());
	
	if(metadata)
	{
	  gd.addRadioButtonGroup("Monitor counter 1",RBlabels,1,3,RBlabels[2]);
	  gd.setInsets(0,20,0);
	  gd.addChoice("select conuter:", counters, "");
	  
	  gd.addRadioButtonGroup("Monitor counter 2",RBlabels,1,3,RBlabels[2]);
	  gd.setInsets(0,20,0);
	  gd.addChoice("select conuter:", counters, "");
	  
	  gd.addRadioButtonGroup("Monitor counter 3",RBlabels,1,3,RBlabels[2]);
	  gd.setInsets(0,20,0);
	  gd.addChoice("select conuter:", counters, "");
	  

	  gd.addRadioButtonGroup("Stack z-Label",RBZlabels,1,3,RBZlabels[0]);
	  gd.setInsets(0,0,0);
	  gd.addChoice("select motor:", motors, "");
	  gd.setInsets(0,0,0);
	  gd.addChoice("select counter:", counters, "");

	  
	  gd.showDialog();
	  if (gd.wasCanceled())
		  return;
	  counter1RB=gd.getNextRadioButton();
	  counter1 = gd.getNextChoiceIndex();
	  counter2RB=gd.getNextRadioButton();
	  counter2 = gd.getNextChoiceIndex();
	  counter3RB=gd.getNextRadioButton();
	  counter3 = gd.getNextChoiceIndex();
	  zRB=gd.getNextRadioButton();
	  zmotor = gd.getNextChoiceIndex();
	  zcounter = gd.getNextChoiceIndex();
	}
	else{
  	  counter1RB="0";
	  counter1 = 0;
	  counter2RB="0";
	  counter2 = 0;
	  counter3RB="0";
	  counter3 = 0;
	  zRB="0";
	  zmotor = 0;
	  zcounter = 0;
	}
	
//	  IJ.write(counter1RB+"   "+counter1);
//	  IJ.write(counter2RB+"   "+counter2);
//	  IJ.write(counter3RB+"   "+counter3);
//	  IJ.write(zRB+"   "+zmotor+"   "+zcounter);

	 ImageStack is=new ImageStack(width,height);
			

           for (int i=0; i<children.length; i++)
          {  IJ.showStatus((i+1)+"/"+children.length);
             if (children[i]==null) return;
             f = new File(directory,children[i]);
          
    // The following code is "borrowed" from ij.plugin.Raw
	  FileInfo fileInfo = new FileInfo();
	  fileInfo.fileFormat = fileInfo.RAW;
	  fileInfo.fileName = children[i];
	  fileInfo.directory = directory;
	  fileInfo.width = width;
	  fileInfo.height = height;
	  fileInfo.offset = offset;
	  fileInfo.nImages = nImages;
	  fileInfo.gapBetweenImages = gapBetweenImages;
	  fileInfo.intelByteOrder = intelByteOrder;
	  fileInfo.whiteIsZero = whiteIsZero;
	  fileInfo.fileType = fileType ;
    
    //Bildobjekt


    // Leave the job to ImageJ for actually reading the image
   //Header untersuchen
    parseESRFDataFormatHeader("EDF", f, fileInfo);

    fileOpener = new FileOpener(fileInfo);
    ImagePlus imgp=fileOpener.open(false);
	//  fileOpener.open();

	
			  double factor=1;
			if(metadata)
			{
			  if (counter1RB==RBlabels[0] ) {
				  int j=counterList.indexOf(counters[counter1]);
				  
				  String val=(String)counterValueList.get(j);
				  factor=factor*(new Double(val)).doubleValue();
//				  IJ.write(counterValueList.toString());
//				  IJ.write(i+"   "+j+"   "+val);
			  }
			  else if (counter1RB==RBlabels[1]) {
				  int j=counterList.indexOf(counters[counter1]);
				  String val=(String)counterValueList.get(j);
				 
				  factor=factor/(new Double(val)).doubleValue();
//				  IJ.write(counterValueList.toString());
//				  IJ.write(i+"   "+j+"   "+val);
			  }
			  
			  if (counter2RB==RBlabels[0] ) {
				  int j=counterList.indexOf(counters[counter2]);
				  
				  String val=(String)counterValueList.get(j);
				  factor=factor*(new Double(val)).doubleValue();
			  }
			  else if (counter2RB==RBlabels[1]) {
				  int j=counterList.indexOf(counters[counter2]);
				  String val=(String)counterValueList.get(j);
				  
				  factor=factor/(new Double(val)).doubleValue();
			  }
			  
			  if (counter3RB==RBlabels[0] ) {
				  int j=counterList.indexOf(counters[counter3]);
				  
				  String val=(String)counterValueList.get(j);
				  factor=factor*(new Double(val)).doubleValue();
			  }
			  else if (counter3RB==RBlabels[1]) {
				  int j=counterList.indexOf(counters[counter3]);
				  String val=(String)counterValueList.get(j);
				  
				  factor=factor/(new Double(val)).doubleValue();
			  }
			}
			  
			  
//    IJ.write(i+"   "+factor);
	
     if(metadata){
	if (zRB==RBZlabels[0] ) {
		is.addSlice(children[i] ,imgp.getChannelProcessor() );
	}		 
	else if (zRB==RBZlabels[1]) {
		int j=motorList.indexOf(motors[zmotor]);
		String val=(String)motorValueList.get(j);
	//	IJ.write("Motor   "+motors[zmotor]+"  "+val);
		is.addSlice(val ,imgp.getChannelProcessor() );
	}	
	else if (zRB==RBZlabels[2]) {
		int j=counterList.indexOf(counters[zcounter]);
		String val=(String)counterValueList.get(j);
	//	IJ.write("Counter   "+zcounter+"  "+j+"  "+counters[zcounter]+"  "+val);
		is.addSlice(val ,imgp.getChannelProcessor() );
        }
	else {
		is.addSlice( imgp.getChannelProcessor() );
	}
      }
        else {
		is.addSlice( imgp.getChannelProcessor() );
	}

    ImageProcessor ip=imgp.getChannelProcessor();
    ip.multiply(factor);
   // is.addSlice( ip );
     } // for

    ImagePlus mystack=new ImagePlus(dir.getName(),is);
    StackWindow sw=new StackWindow(mystack);
  

	
}

 private void parseESRFDataFormatHeader(String type, File f, FileInfo fileInfo)
  {
	  counterList=new ArrayList();
	  counterValueList=new ArrayList();
	  motorList=new ArrayList();
	  motorValueList=new ArrayList();
	  
	  RandomAccessFile in;
    char h;
    int headerSize, headerStart, noBrackets, i, iParam = 0;
    double dParam = 0.0;
    byte[] header;
    String key, param, token, headerString;
    String  oldtoken=new String("");

    try
    {
      in = new RandomAccessFile(f, "r");
      headerSize = 0;
      noBrackets = 0;
      do
      {
        headerStart = headerSize;
        in.seek(headerStart);
        do
        {
          h = getChar(in);
          headerSize += 1;
          if (h == '{') noBrackets++;
          else if (h== '}') noBrackets--;
        } while (noBrackets != 0);
        //  Read header
        header = new byte[headerSize-headerStart+1];
        in.seek(headerStart);
        in.readFully(header);
        headerString = new String(header);
      } while ( type.equals("EHF") && (headerString.indexOf("EDF_DataBlockID") < 0) );
    }
    catch (IOException ex)
    {
     // IJ.write("IOException caught: "+ex);
      return;
    }
	 // if (IJ.debugMode) IJ.write("ImportDialog: "+fileInfo)
;

    // Set offset
    fileInfo.offset = headerSize+1;

    // Extract information from header
    StringTokenizer st = new StringTokenizer ( headerString, ";" );
        token=new String("");
	  while (st.hasMoreTokens()) {
		oldtoken=new String(token);
	    token = st.nextToken();
	    i = token.indexOf("=");
	    if (i<=0) continue;
	    key = token.substring(1,i-1).trim();
	    param = token.substring(i+1).trim();
 	 if (key.equals("counter_mne"))  
	  { //IJ.write(key+": "+param);
		  StringTokenizer ts = new StringTokenizer ( param);
		  while (ts.hasMoreTokens()) 
		  {
			  String s = ts.nextToken();
			  counterList.add(s);				  
		  }
	  }
	  else if (key.equals("counter_pos"))  
	  { //IJ.write(key+": "+param);
		  StringTokenizer ts = new StringTokenizer ( param);
		  while (ts.hasMoreTokens()) 
		  {
			  String s = ts.nextToken();
			  counterValueList.add(s);				  
		  }
	  }
	  else if (key.equals("motor_mne"))  
	  { //IJ.write(key+": "+param);
		  StringTokenizer ts = new StringTokenizer ( param);
		  while (ts.hasMoreTokens()) 
		  {
			  String s = ts.nextToken();
			  motorList.add(s);				  
		  }
	  }
	  else if (key.equals("motor_pos"))  
	  { //IJ.write(key+": "+param);
		  StringTokenizer ts = new StringTokenizer ( param);
		  while (ts.hasMoreTokens()) 
		  {
			  String s = ts.nextToken();
			  motorValueList.add(s);				  
		  }
	  }
	  else 
               //IJ.write(key+": "+param)
                 ;

      // For debugging - output to stdout is persistent:
      // System.out.println("DOING |"+key+"|: |" + param+"|");

	    try
      {
        iParam = Integer.valueOf(param).intValue();
		    dParam = Integer.valueOf(param).doubleValue();
	    }
	    catch(NumberFormatException numberformatexception) {};
	    if (key.equals("EDF_BinaryFileName"))
      {
        fileInfo.fileName = param;
        System.out.println("DEBUG: "+key+" = "+param);
        continue;
      }
	    if (key.equals("EDF_BinaryFilePosition"))
      {
        fileInfo.offset = iParam;
        continue;
      }
	    if (key.equals("Dim_1"))
      {
        fileInfo.width = iParam;
        continue;
      }
	    if (key.equals("Dim_2"))
      {
        fileInfo.height = iParam;
        continue;
      }
      if (key.equals("DataType"))  
      {
		    if (param.equals("UnsignedLong") || param.equals("UnsignedInteger")|| param.equals("SignedLong"))  // SignedLong added in 2014
        {
		      fileInfo.fileType = fileInfo.GRAY32_INT;
        }
		    else if (param.equals("UnsignedShort"))
        {
		      fileInfo.fileType = fileInfo.GRAY16_UNSIGNED;
        }
		    else if (param.equals("Float") || param.equals("FloatValue"))
        {
          fileInfo.fileType = fileInfo.GRAY32_FLOAT;
        }
		    else if (param.equals("UnsignedByte") || param.equals("UnsignedChar"))
	{
		      fileInfo.fileType = fileInfo.GRAY8;
	}
		    else if (param.equals("SignedShort"))
	{
		      fileInfo.fileType = fileInfo.GRAY16_SIGNED;
	}
		    else
        {
         // IJ.write("WARNING: unknown data type " + param)
;
        }
		    continue;
      }
      if (key.equals("ByteOrder"))
      {
        fileInfo.intelByteOrder = param.equals("LowByteFirst");
      }
    }
    return;
  }
 char getChar(RandomAccessFile in) throws IOException
  {
	  int b = in.read();
	  return (char)b;
  }
}



