/*
***********************************************************************
* Written and maintained by Linus Pithan 			      *
* (linus.pithan@physik.hu-berlin.de)                                  *   
*                                                                     *
* Changes:                                                            *
*                                                                     *
* 15.04.2015  Finalisation of version 1'st beta version               *
*                                                                     *
***********************************************************************
*/

//import ij.IJ;
import ij.*;
import ij.plugin.*;
import ij.process.*;
import ij.gui.*;
//import ij.ImagePlus;
import ij.gui.PolygonRoi;
import ij.gui.Roi;
import ij.plugin.PlugIn;
import ij.plugin.filter.*;


//import ij.plugin.filter.RGBStackSplitter;

import ij.measure.*;

//import java.awt.*;

public class Moving_ROI_through_Stack_ implements PlugIn 
{
 private String fileName;
 private static int stacksize;
 private static int width=0;
 private static int height=0;
 private static int bitDepth=0;
 private static double max=0;
 private static double min=0;
 int xa;
 int ya;
 int xe;
 int ye;
 int dx=10;
 int dy=10; 
 int sleeptime=100;

    public void run(String arg) {
        ImagePlus imp = IJ.getImage();
    if (imp.getStackSize()==1)
    {IJ.error("Stack required"); return;}

   


    fileName=imp.getTitle();
    ImageStack stack = imp.getStack();
    stacksize=stack.getSize();
    width = stack.getWidth();
    height = stack.getHeight();

    ImageProcessor ip = stack.getProcessor(1);
    min=ip.getMin();
    max=ip.getMax();
    if (ip instanceof ByteProcessor)
			bitDepth = 8;
		else if (ip instanceof ShortProcessor)
			bitDepth = 16;
		else if (ip instanceof FloatProcessor)
			bitDepth = 32;
		else
			throw new IllegalArgumentException("RGB stacks not supported");
   

String ErrorMessage = new String("One of your values was not properly formatted.\nThe default values will be used.");
GenericDialog d = new GenericDialog("Moving ROI through Stack");             
		
                d.addNumericField("x start position:", xa, 0);
                d.addNumericField("y start position:", ya, 0);
                d.addNumericField("x final position:", width, 0);
                d.addNumericField("y final position:", height, 0);
                d.addNumericField("box width x:", dx, 0);
                d.addNumericField("box width y:", dy, 0);
                d.addNumericField("stack length:", stacksize, 0);
                d.addNumericField("width:",  width, 0);
                d.addNumericField("height:", height, 0);
//                d.addNumericField("bitDepth:", bitDepth, 0);
//                d.addNumericField("maximum:", max, 0);
//                d.addNumericField("minimum:", min, 0);
	        d.addNumericField("Sleeptime in ms:", sleeptime, 0);
		d.showDialog();
            if(d.wasCanceled()) return;

 xa=(int)Math.round(d.getNextNumber());
 ya=(int)Math.round(d.getNextNumber());
 xe=(int)Math.round(d.getNextNumber());
 ye=(int)Math.round(d.getNextNumber());
 dx=(int)Math.round(d.getNextNumber());
 dy=(int)Math.round(d.getNextNumber());

 if(d.invalidNumber()) {IJ.error("Invalid input Number"); return;}
  
  ResultsTable rt=new ResultsTable();
  Analyzer a=new Analyzer(imp,Measurements.INTEGRATED_DENSITY,rt);
  for (int i=1; i<= stacksize;i++)
 {
   ip = stack.getProcessor(i);
  imp.setSlice(i);
  IJ.wait(sleeptime);
  int x=xa+((xe-xa)*i)/stacksize-dx/2;
  int y=ya+((ye-ya)*i)/stacksize-dy/2;
  imp.setRoi(new Roi(x,y,dx,dy));
  a.measure();
  } 
  rt.show(fileName+"  Measurements.INTEGRATED_DENSITY");    
    }

}
