/*
***********************************************************************
* Written and maintained by Linus Pithan 			      *
* (linus.pithan@physik.hu-berlin.de)                                  *   
*                                                                     *
* Changes:                                                            *
*                                                                     *
* 15.04.2015  Finalisation of version 1'st beta version               *
*                                                                     *
***********************************************************************
*/

import ij.*;
import ij.plugin.*;
import ij.plugin.filter.RGBStackSplitter;
import ij.process.*;
import ij.gui.*;
import java.awt.*;

public class Stack_Stitcher implements PlugIn 
{

private static int stacksize;
private static int width=0;
private static int height=0;
private int dimension = 0;
private int bitDepth = 0;
private int v = 0;
private double min=1000000.0;
private double max=0.0;
private String fileName;
boolean ueberlagerungstest=false;
java.util.Vector ch;
public void run (String arg) {
		ImagePlus imp = IJ.getImage();
		if (imp.getStackSize()==1)
			{IJ.error("Stack required"); return;}
                fileName=imp.getTitle();
                ImageStack stack = imp.getStack();
                 stacksize=stack.getSize();
                 width = stack.getWidth();
		 height = stack.getHeight();
                 dimension = width*height;
                float[][] stackslice = new float[stacksize][dimension];
                ImageProcessor ip = stack.getProcessor(1);
                min=ip.getMin();
                max=ip.getMax();
                if (ip instanceof ByteProcessor)
			bitDepth = 8;
		else if (ip instanceof ShortProcessor)
			bitDepth = 16;
		else if (ip instanceof FloatProcessor)
			bitDepth = 32;
		else
			throw new IllegalArgumentException("RGB stacks not supported");

		String ErrorMessage = new String("One of your values was not properly formatted.\nThe default values will be used.");
		GenericDialog d = new GenericDialog("Stack Stitcher");
                d.addNumericField("Overlap in px:", v, 0);
		d.addMessage("--------------------");
                d.addNumericField("Brightness max.:", max, 0);
                d.addNumericField("Brightness min.:", min, 0);
		d.addNumericField("Stacksize:", stacksize, 0);
                d.addNumericField("Width:",  width, 0);
                d.addNumericField("Height:", height, 0);
                d.addNumericField("BitDepth:", bitDepth, 0);
                //d.addCheckbox("Ueberlagerungstest:", false);

		
		d.showDialog();
		if(d.wasCanceled()) return;
               v=(int)Math.round(d.getNextNumber());
               max=d.getNextNumber();
               min=d.getNextNumber();
              ueberlagerungstest=d.getNextBoolean();
		//double percentvar = d.getNextNumber();
		//gain = d.getNextNumber();
               	
		if(d.invalidNumber())
			{IJ.error("Invalid input Number"); return;}
		if(bitDepth!=32|v<0|v>width)
                        {
			IJ.error(ErrorMessage);
			
                        v=0;
		       }

		   // hier muss etwas passieren
                   //  int i=0; // erstes stackslice
                    for (int i=0;i<stacksize;i++)
                   stackslice[i] = toFloat(stack.getPixels(i+1), bitDepth);
               
                  //createImageTest(width,height,v,fileName,stackslice);
              
             createImageTest(width,height,v,fileName,stackslice);
	      }

public void createImageTest(int w,int h,int v,String filename,float[][] array)
{ String vString= (new Integer(v)).toString();
  int[][] sliceUeberlagerung=new int[stacksize][2];
  int[] ueberlagerung=new int[(w+(w-v)*(stacksize-1))];
int ueMax=0;

GenericDialog d = new GenericDialog("Positions");
                //d.addNumericField("ueMax:", ueMax, 0); // erst unten hinzufügen
                d.addTextAreas("", null,10,30);                 
                TextArea textArea=d.getTextArea1();  
      
 // großes Bild
           // ImagePlus imp = IJ.createImage(filename+" v="+vString, "32-bit black", w*stacksize, h, 1);
           //  imp.show();
           //  IJ.wait(2000);

           float[] data = new float[h*w*stacksize];
            for (int s=0;s<stacksize;s++)
           { 
            textArea.append("slice: "+s+"  xmin="+(w-v)*s+ "  xmax="+ ((w-v)*s+w)+ "\n") ;
            sliceUeberlagerung[s][0]=(w-v)*s;
            sliceUeberlagerung[s][1]=((w-v)*s+w);
           for (int  j=0; j<h; j++)
           for ( int i=0; i<w; i++)
          {  int l=j*w+i;
             int  k=j*w*stacksize+i+(w-v)*s;
              data[k] =  data[k]+array[s][l];
           
            
          }
       }
       // wenn die ursprüngliche Montage gezeigt werden soll
       //    ImageProcessor ip = imp.getProcessor();
        //   ip.setPixels(data);
       //    ip.setMinAndMax(min,max);
       //    imp.updateAndDraw();

// kleines Bild
        
          ImagePlus imp1 = IJ.createImage(filename+" v="+vString, "32-bit black", w+(w-v)*(stacksize-1), h, 1);
             imp1.show();
       
          float[] data1 = new float[h*(w+(w-v)*(stacksize-1))];
           int k=-1;
           int l=-1;

           for ( int j=0; j<h; j++)
          { if(j>0) {k=k+v*(stacksize-1);  }
           for ( int i=0; i<(w+(w-v)*(stacksize-1)); i++)  
           { if( i<(w+(w-v)*(stacksize-1))) {k=k+1; l=l+1;}
            
            //if((l< h*(w+(w-v)*(stacksize-1))) & (k<h*w*stacksize))
           {
            data1[l]=data[k];}
            }
           }

// hier Brightness-Anpassung berechnen
 for (int s=0;s<stacksize;s++)
{ int a=sliceUeberlagerung[s][0]; //nfang des Überlagerungsbereichs
  int e=sliceUeberlagerung[s][1]; //Ende des Überlagerungsbereichs
 for (int i=a;i<e;i++)
 ueberlagerung[i]=ueberlagerung[i]+1;
 }
// Maximum der Überlagerungen berechnen
for(int i=0;i<(w+(w-v)*(stacksize-1));i++)
{ if(ueMax< ueberlagerung[i]) ueMax= ueberlagerung[i];
}
// jetzt Brightness-Anpassung durchführen
for ( int i=0; i<(w+(w-v)*(stacksize-1)); i++) 
{ float factor=(ueMax*1.0f)/ueberlagerung[i];
  for( int j=0;j< h;j++)
{ 
 int adresse= j*(w+(w-v)*(stacksize-1))+i;  
if(ueberlagerungstest) data1[adresse]=ueberlagerung[i]*10.0f; else
  data1[adresse]=factor*data1[adresse];
// data1[adresse]=ueberlagerung[i]*10.0f;  // für Streifentest
  }
 }

           

 d.addNumericField("max nuber of layers:", ueMax, 0);//fauler Trick



// wenn die  Montage mit Verschiebung gezeigt werden soll
          ImageProcessor ip1 = imp1.getProcessor();
           ip1.setPixels(data1);
           //ip1.setMinAndMax(min,max);
           ip1.setMinAndMax(min,ueMax*max);
           imp1.updateAndDraw();

           
                 
    d.showDialog(); 

}

{
}
public float[] toFloat(Object array, int bitDepth) 
{float[] floats = null;
                   switch (bitDepth) {
			case 8:
				byte[] bytes = (byte[])array;
				floats = new float[bytes.length];
				for(int i=0; i<floats.length; i++)
					floats[i] = (bytes[i]&0xff);
				break;
			case 16:
				short[] shorts = (short[])array;
				floats = new float[shorts.length];
				for(int i=0; i<floats.length; i++)
					floats[i] = (shorts[i]&0xffff);
				break;
			case 32:
				 floats = (float[])array;
				
				break;
		}
		return floats;
}
    }
